@@ -1180,6 +1180,27 @@ exports.gh129 = function() {
   (span.parentNode === null).should.equal(true);
 };
 
+exports.gh135 = function() {
+  var document = domino.createDocument('<a target="foobar"></a>');
+  var a = document.querySelectorAll('*[target$="aaaaaaa"]');
+  a.length.should.equal(0);
+};
+
+exports.gh136 = function() {
+  var document = domino.createDocument('<option> \f\tabc\n');
+  var o = document.querySelector('option');
+  o.value.should.equal('abc');
+  o.text.should.equal('abc');
+  o.textContent.should.equal(' \f\tabc\n');
+  o.value = ' def ';
+  o.value.should.equal(' def ');
+  o.text.should.equal('abc');
+  o.text = ' ghi ';
+  o.text.should.equal('ghi');
+  o.textContent.should.equal(' ghi ');
+  o.outerHTML.should.equal("<option value=\" def \"> ghi </option>");
+};
+
 // Examples from
 // https://developer.mozilla.org/en-US/docs/Web/API/Element/outerHTML
 exports.outerHTML1 = function() {
@@ -1272,6 +1293,14 @@ exports.insertForeignElement = function() {
   );
 };
 
+exports.bodyNL = function() {
+  var document = domino.createDocument(
+    '<body></body>\n'
+  );
+  // This may be surprising, but it's what the spec requires.
+  document.body.textContent.should.equal('\n');
+};
+
 exports.testIncompleteTag = function() {
   var document = domino.createDocument('<p>hello<');
   document.body.outerHTML.should.equal('<body><p>hello&lt;</p></body>');
